//--------------------------------------------------------------------
//
//  Modul test_res - Version mit variabel programmierbarer Aufloesung
//
//  Testumgebung fuer den Display-Controller
//
//--------------------------------------------------------------------

`include "discount_defs.v"

module test_res;

  `define Simtime ((`Hline)*(`Vframe)*120)  // 1.2 Frames simulieren

  // Deklarationen
  reg [`Bsz-1:0]  MEM[0:`Msz-1];  // Bildspeicher
  reg [`Bsz-1:0]  TEMP;           // Hilfsvariable (Speicher)
  reg [`Dsz-1:0]  DATA;           // Bildspeicherausgang
  reg             CLOCK,          // Takt
                  NRST,           // Initialisierung
                  WE,             // Write-Enable Programmierbus
                  SNAPSHOT;       // Bild speichern (PBM)
  reg  [`Asz-1:0] PADDR;          // Programmier-Adressbus
  wire [`Rsz-1:0] PDATA;          // Bidirektionaler Programmierdatenbus
  reg  [`Rsz-1:0] PDATA_REG;      // Bidirektionaler Programmierdatenbus
  wire [`Asz-1:0] ADDR;           // Bildspeicheradresse
  wire            PIXEL,          // Bildpunkt
                  CSYNC,          // kombinierte Synchronisation
                  HSYNC,          // horizontale Synchronisation
                  VSYNC;          // vertikale Synchronisation
  integer         BYTE, BIT,      // Speicherposition
                  H, V;           // Bildposition

  //
  // Testinstanz des Display-Controllers
  //
  discount_res DISCOUNT (ADDR,PIXEL,CSYNC,HSYNC,VSYNC,DATA,
                         CLOCK,NRST,PADDR,WE,PDATA);

  //
  // Instanz des Testmonitors (schreibt PBM)
  //
  crt CRT (PIXEL,CSYNC,HSYNC,VSYNC,SNAPSHOT);

  //
  // Bidirektionalen Bus treiben
  assign PDATA = PDATA_REG;

  //
  // Speicher initialisieren, weisse Diagonale
  //
  initial begin
    BYTE = 0;                             // Initialisierung
    BIT = `Bsz-1;
    TEMP = MEM[0];
    for (V = 0; V < 200; V = V+1)         // Zeilen <- Anpassen!
      for (H = 0; H < 320; H = H+1) begin // Punkte <- Anpassen!
        if (V == H) TEMP[BIT] = 1;        // Diagonale
        else        TEMP[BIT] = 0;
        if (BIT == 0) begin               // neues Byte
          MEM[BYTE] = TEMP;
          if (BYTE < `Msz-1)              // nicht letztes Byte
            BYTE = BYTE+1;
          TEMP = MEM[BYTE];
          BIT = `Bsz-1;
        end
        else BIT = BIT-1;
      end
  end

  //
  // Takt erzeugen
  //
  always begin
    CLOCK = 1'b0;
    #50;
    CLOCK = 1'b1;
    #50;
  end

  //
  // Reset
  //
  initial begin
    NRST = 1'b0;
    @(negedge CLOCK);
    NRST = 1'b1;
  end

  //
  // Bildspeicherdaten auslesen
  //
  always @(ADDR) begin
    DATA = MEM[ADDR];
  end

  initial
    #`Simtime $stop;                    // Simulationsdauer

  //
  // Textausgabe
  //
  initial
    $monitor("Takt=%d  PIXEL=%b  HSYNC=%b  VSYNC=%b",
      $time/100, PIXEL, HSYNC, VSYNC);

  //
  // Diagonale verifizieren
  //
  always @(posedge CLOCK) begin
    H = DISCOUNT.HCOUNT.HCNT-DISCOUNT.HCOUNT.H1-1;      // Bildposition
    V = DISCOUNT.VCOUNT.VCNT-DISCOUNT.VCOUNT.V1;

    if ((H>=0) && (H< DISCOUNT.HCOUNT.H) && // echter Bildpunkt
        (V> 0) && (V<=DISCOUNT.VCOUNT.V))

      if (((H==V) && (PIXEL==0)) ||      // Diagonale
          ((H!=V) && (PIXEL==1)))        // ausserhalb Diagonale
        $display(
        "falsche Diagonale bei Takt %0d, PIXEL = %b, H = %0d, V = %0d"
        ,$time/100, PIXEL, H, V);
  end

  //
  // Snapshot speichern
  //
  initial
    SNAPSHOT = 1;

  //
  // Aufloesung programmieren
  //
  initial begin
    PADDR = 0; WE = 0; PDATA_REG = `Rsz'bz;
    #200;
    @(negedge CLOCK);
    WE = 1; PDATA_REG = 48; #100;
    PADDR = 1; PDATA_REG = 320; #100;
    PADDR = 2; PDATA_REG = 44; #100;
    PADDR = 3; #100;
    PADDR = 4; PDATA_REG = 60; #100;
    PADDR = 5; PDATA_REG = 200; #100;
    PADDR = 6; PDATA_REG = 50; #100;
    PADDR = 7; PDATA_REG = 16; #100;
    // Wieder auslesen
    WE = 0; PDATA_REG = `Rsz'bz; PADDR = 1; #100;
    $display("Neue Aufloesung: x %d", PDATA);
    PADDR = 5; #100;
    $display("Neue Aufloesung: y %d", PDATA);
  end

endmodule // test_res
